# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: folder.proto
"""Generated protocol buffer code."""
from google.protobuf.internal import enum_type_wrapper
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import message as _message
from google.protobuf import reflection as _reflection
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()




DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n\x0c\x66older.proto\x12\x06\x46older\"\\\n\x10\x45ncryptedDataKey\x12\x14\n\x0c\x65ncryptedKey\x18\x01 \x01(\x0c\x12\x32\n\x10\x65ncryptedKeyType\x18\x02 \x01(\x0e\x32\x18.Folder.EncryptedKeyType\"\x82\x01\n\x16SharedFolderRecordData\x12\x11\n\tfolderUid\x18\x01 \x01(\x0c\x12\x11\n\trecordUid\x18\x02 \x01(\x0c\x12\x0e\n\x06userId\x18\x03 \x01(\x05\x12\x32\n\x10\x65ncryptedDataKey\x18\x04 \x03(\x0b\x32\x18.Folder.EncryptedDataKey\"\\\n\x1aSharedFolderRecordDataList\x12>\n\x16sharedFolderRecordData\x18\x01 \x03(\x0b\x32\x1e.Folder.SharedFolderRecordData\"_\n\x15SharedFolderRecordFix\x12\x11\n\tfolderUid\x18\x01 \x01(\x0c\x12\x11\n\trecordUid\x18\x02 \x01(\x0c\x12 \n\x18\x65ncryptedRecordFolderKey\x18\x03 \x01(\x0c\"Y\n\x19SharedFolderRecordFixList\x12<\n\x15sharedFolderRecordFix\x18\x01 \x03(\x0b\x32\x1d.Folder.SharedFolderRecordFix\"\xa2\x02\n\rRecordRequest\x12\x11\n\trecordUid\x18\x01 \x01(\x0c\x12&\n\nrecordType\x18\x02 \x01(\x0e\x32\x12.Folder.RecordType\x12\x12\n\nrecordData\x18\x03 \x01(\x0c\x12\x1a\n\x12\x65ncryptedRecordKey\x18\x04 \x01(\x0c\x12&\n\nfolderType\x18\x05 \x01(\x0e\x32\x12.Folder.FolderType\x12\x12\n\nhowLongAgo\x18\x06 \x01(\x03\x12\x11\n\tfolderUid\x18\x07 \x01(\x0c\x12 \n\x18\x65ncryptedRecordFolderKey\x18\x08 \x01(\x0c\x12\r\n\x05\x65xtra\x18\t \x01(\x0c\x12\x15\n\rnonSharedData\x18\n \x01(\x0c\x12\x0f\n\x07\x66ileIds\x18\x0b \x03(\x03\"E\n\x0eRecordResponse\x12\x11\n\trecordUid\x18\x01 \x01(\x0c\x12\x10\n\x08revision\x18\x02 \x01(\x03\x12\x0e\n\x06status\x18\x03 \x01(\t\"\x80\x01\n\x12SharedFolderFields\x12\x1b\n\x13\x65ncryptedFolderName\x18\x01 \x01(\x0c\x12\x13\n\x0bmanageUsers\x18\x02 \x01(\x08\x12\x15\n\rmanageRecords\x18\x03 \x01(\x08\x12\x0f\n\x07\x63\x61nEdit\x18\x04 \x01(\x08\x12\x10\n\x08\x63\x61nShare\x18\x05 \x01(\x08\"3\n\x18SharedFolderFolderFields\x12\x17\n\x0fsharedFolderUid\x18\x01 \x01(\x0c\"\x8f\x02\n\rFolderRequest\x12\x11\n\tfolderUid\x18\x01 \x01(\x0c\x12&\n\nfolderType\x18\x02 \x01(\x0e\x32\x12.Folder.FolderType\x12\x17\n\x0fparentFolderUid\x18\x03 \x01(\x0c\x12\x12\n\nfolderData\x18\x04 \x01(\x0c\x12\x1a\n\x12\x65ncryptedFolderKey\x18\x05 \x01(\x0c\x12\x36\n\x12sharedFolderFields\x18\x06 \x01(\x0b\x32\x1a.Folder.SharedFolderFields\x12\x42\n\x18sharedFolderFolderFields\x18\x07 \x01(\x0b\x32 .Folder.SharedFolderFolderFields\"E\n\x0e\x46olderResponse\x12\x11\n\tfolderUid\x18\x01 \x01(\x0c\x12\x10\n\x08revision\x18\x02 \x01(\x03\x12\x0e\n\x06status\x18\x03 \x01(\t\"w\n\x19ImportFolderRecordRequest\x12,\n\rfolderRequest\x18\x01 \x03(\x0b\x32\x15.Folder.FolderRequest\x12,\n\rrecordRequest\x18\x02 \x03(\x0b\x32\x15.Folder.RecordRequest\"|\n\x1aImportFolderRecordResponse\x12.\n\x0e\x66olderResponse\x18\x01 \x03(\x0b\x32\x16.Folder.FolderResponse\x12.\n\x0erecordResponse\x18\x02 \x03(\x0b\x32\x16.Folder.RecordResponse\"\xee\x01\n\x18SharedFolderUpdateRecord\x12\x11\n\trecordUid\x18\x01 \x01(\x0c\x12\x17\n\x0fsharedFolderUid\x18\x02 \x01(\x0c\x12\x0f\n\x07teamUid\x18\x03 \x01(\x0c\x12(\n\x07\x63\x61nEdit\x18\x04 \x01(\x0e\x32\x17.Folder.SetBooleanValue\x12)\n\x08\x63\x61nShare\x18\x05 \x01(\x0e\x32\x17.Folder.SetBooleanValue\x12\x1a\n\x12\x65ncryptedRecordKey\x18\x06 \x01(\x0c\x12\x10\n\x08revision\x18\x07 \x01(\x05\x12\x12\n\nexpiration\x18\x08 \x01(\x12\"\xb5\x01\n\x16SharedFolderUpdateUser\x12\x10\n\x08username\x18\x01 \x01(\t\x12,\n\x0bmanageUsers\x18\x02 \x01(\x0e\x32\x17.Folder.SetBooleanValue\x12.\n\rmanageRecords\x18\x03 \x01(\x0e\x32\x17.Folder.SetBooleanValue\x12\x17\n\x0fsharedFolderKey\x18\x04 \x01(\x0c\x12\x12\n\nexpiration\x18\x05 \x01(\x12\"\x82\x01\n\x16SharedFolderUpdateTeam\x12\x0f\n\x07teamUid\x18\x01 \x01(\x0c\x12\x13\n\x0bmanageUsers\x18\x02 \x01(\x08\x12\x15\n\rmanageRecords\x18\x03 \x01(\x08\x12\x17\n\x0fsharedFolderKey\x18\x04 \x01(\x0c\x12\x12\n\nexpiration\x18\x05 \x01(\x12\"\x8e\x07\n\x1bSharedFolderUpdateV3Request\x12,\n$sharedFolderUpdateOperation_dont_use\x18\x01 \x01(\x05\x12\x17\n\x0fsharedFolderUid\x18\x02 \x01(\x0c\x12!\n\x19\x65ncryptedSharedFolderName\x18\x03 \x01(\x0c\x12\x10\n\x08revision\x18\x04 \x01(\x03\x12\x13\n\x0b\x66orceUpdate\x18\x05 \x01(\x08\x12\x13\n\x0b\x66romTeamUid\x18\x06 \x01(\x0c\x12\x33\n\x12\x64\x65\x66\x61ultManageUsers\x18\x07 \x01(\x0e\x32\x17.Folder.SetBooleanValue\x12\x35\n\x14\x64\x65\x66\x61ultManageRecords\x18\x08 \x01(\x0e\x32\x17.Folder.SetBooleanValue\x12/\n\x0e\x64\x65\x66\x61ultCanEdit\x18\t \x01(\x0e\x32\x17.Folder.SetBooleanValue\x12\x30\n\x0f\x64\x65\x66\x61ultCanShare\x18\n \x01(\x0e\x32\x17.Folder.SetBooleanValue\x12?\n\x15sharedFolderAddRecord\x18\x0b \x03(\x0b\x32 .Folder.SharedFolderUpdateRecord\x12;\n\x13sharedFolderAddUser\x18\x0c \x03(\x0b\x32\x1e.Folder.SharedFolderUpdateUser\x12;\n\x13sharedFolderAddTeam\x18\r \x03(\x0b\x32\x1e.Folder.SharedFolderUpdateTeam\x12\x42\n\x18sharedFolderUpdateRecord\x18\x0e \x03(\x0b\x32 .Folder.SharedFolderUpdateRecord\x12>\n\x16sharedFolderUpdateUser\x18\x0f \x03(\x0b\x32\x1e.Folder.SharedFolderUpdateUser\x12>\n\x16sharedFolderUpdateTeam\x18\x10 \x03(\x0b\x32\x1e.Folder.SharedFolderUpdateTeam\x12 \n\x18sharedFolderRemoveRecord\x18\x11 \x03(\x0c\x12\x1e\n\x16sharedFolderRemoveUser\x18\x12 \x03(\t\x12\x1e\n\x16sharedFolderRemoveTeam\x18\x13 \x03(\x0c\x12\x19\n\x11sharedFolderOwner\x18\x14 \x01(\t\"C\n\x1eSharedFolderUpdateRecordStatus\x12\x11\n\trecordUid\x18\x01 \x01(\x0c\x12\x0e\n\x06status\x18\x02 \x01(\t\"@\n\x1cSharedFolderUpdateUserStatus\x12\x10\n\x08username\x18\x01 \x01(\t\x12\x0e\n\x06status\x18\x02 \x01(\t\"?\n\x1cSharedFolderUpdateTeamStatus\x12\x0f\n\x07teamUid\x18\x01 \x01(\x0c\x12\x0e\n\x06status\x18\x02 \x01(\t\"\xdf\x05\n\x1cSharedFolderUpdateV3Response\x12\x10\n\x08revision\x18\x01 \x01(\x03\x12K\n\x1bsharedFolderAddRecordStatus\x18\x02 \x03(\x0b\x32&.Folder.SharedFolderUpdateRecordStatus\x12G\n\x19sharedFolderAddUserStatus\x18\x03 \x03(\x0b\x32$.Folder.SharedFolderUpdateUserStatus\x12G\n\x19sharedFolderAddTeamStatus\x18\x04 \x03(\x0b\x32$.Folder.SharedFolderUpdateTeamStatus\x12N\n\x1esharedFolderUpdateRecordStatus\x18\x05 \x03(\x0b\x32&.Folder.SharedFolderUpdateRecordStatus\x12J\n\x1csharedFolderUpdateUserStatus\x18\x06 \x03(\x0b\x32$.Folder.SharedFolderUpdateUserStatus\x12J\n\x1csharedFolderUpdateTeamStatus\x18\x07 \x03(\x0b\x32$.Folder.SharedFolderUpdateTeamStatus\x12N\n\x1esharedFolderRemoveRecordStatus\x18\x08 \x03(\x0b\x32&.Folder.SharedFolderUpdateRecordStatus\x12J\n\x1csharedFolderRemoveUserStatus\x18\t \x03(\x0b\x32$.Folder.SharedFolderUpdateUserStatus\x12J\n\x1csharedFolderRemoveTeamStatus\x18\n \x03(\x0b\x32$.Folder.SharedFolderUpdateTeamStatus*\x1a\n\nRecordType\x12\x0c\n\x08password\x10\x00*^\n\nFolderType\x12\x12\n\x0e\x64\x65\x66\x61ult_folder\x10\x00\x12\x0f\n\x0buser_folder\x10\x01\x12\x11\n\rshared_folder\x10\x02\x12\x18\n\x14shared_folder_folder\x10\x03*V\n\x10\x45ncryptedKeyType\x12\n\n\x06no_key\x10\x00\x12\x19\n\x15\x65ncrypted_by_data_key\x10\x01\x12\x1b\n\x17\x65ncrypted_by_public_key\x10\x02*M\n\x0fSetBooleanValue\x12\x15\n\x11\x42OOLEAN_NO_CHANGE\x10\x00\x12\x10\n\x0c\x42OOLEAN_TRUE\x10\x01\x12\x11\n\rBOOLEAN_FALSE\x10\x02\x42\"\n\x18\x63om.keepersecurity.protoB\x06\x46olderb\x06proto3')

_RECORDTYPE = DESCRIPTOR.enum_types_by_name['RecordType']
RecordType = enum_type_wrapper.EnumTypeWrapper(_RECORDTYPE)
_FOLDERTYPE = DESCRIPTOR.enum_types_by_name['FolderType']
FolderType = enum_type_wrapper.EnumTypeWrapper(_FOLDERTYPE)
_ENCRYPTEDKEYTYPE = DESCRIPTOR.enum_types_by_name['EncryptedKeyType']
EncryptedKeyType = enum_type_wrapper.EnumTypeWrapper(_ENCRYPTEDKEYTYPE)
_SETBOOLEANVALUE = DESCRIPTOR.enum_types_by_name['SetBooleanValue']
SetBooleanValue = enum_type_wrapper.EnumTypeWrapper(_SETBOOLEANVALUE)
password = 0
default_folder = 0
user_folder = 1
shared_folder = 2
shared_folder_folder = 3
no_key = 0
encrypted_by_data_key = 1
encrypted_by_public_key = 2
BOOLEAN_NO_CHANGE = 0
BOOLEAN_TRUE = 1
BOOLEAN_FALSE = 2


_ENCRYPTEDDATAKEY = DESCRIPTOR.message_types_by_name['EncryptedDataKey']
_SHAREDFOLDERRECORDDATA = DESCRIPTOR.message_types_by_name['SharedFolderRecordData']
_SHAREDFOLDERRECORDDATALIST = DESCRIPTOR.message_types_by_name['SharedFolderRecordDataList']
_SHAREDFOLDERRECORDFIX = DESCRIPTOR.message_types_by_name['SharedFolderRecordFix']
_SHAREDFOLDERRECORDFIXLIST = DESCRIPTOR.message_types_by_name['SharedFolderRecordFixList']
_RECORDREQUEST = DESCRIPTOR.message_types_by_name['RecordRequest']
_RECORDRESPONSE = DESCRIPTOR.message_types_by_name['RecordResponse']
_SHAREDFOLDERFIELDS = DESCRIPTOR.message_types_by_name['SharedFolderFields']
_SHAREDFOLDERFOLDERFIELDS = DESCRIPTOR.message_types_by_name['SharedFolderFolderFields']
_FOLDERREQUEST = DESCRIPTOR.message_types_by_name['FolderRequest']
_FOLDERRESPONSE = DESCRIPTOR.message_types_by_name['FolderResponse']
_IMPORTFOLDERRECORDREQUEST = DESCRIPTOR.message_types_by_name['ImportFolderRecordRequest']
_IMPORTFOLDERRECORDRESPONSE = DESCRIPTOR.message_types_by_name['ImportFolderRecordResponse']
_SHAREDFOLDERUPDATERECORD = DESCRIPTOR.message_types_by_name['SharedFolderUpdateRecord']
_SHAREDFOLDERUPDATEUSER = DESCRIPTOR.message_types_by_name['SharedFolderUpdateUser']
_SHAREDFOLDERUPDATETEAM = DESCRIPTOR.message_types_by_name['SharedFolderUpdateTeam']
_SHAREDFOLDERUPDATEV3REQUEST = DESCRIPTOR.message_types_by_name['SharedFolderUpdateV3Request']
_SHAREDFOLDERUPDATERECORDSTATUS = DESCRIPTOR.message_types_by_name['SharedFolderUpdateRecordStatus']
_SHAREDFOLDERUPDATEUSERSTATUS = DESCRIPTOR.message_types_by_name['SharedFolderUpdateUserStatus']
_SHAREDFOLDERUPDATETEAMSTATUS = DESCRIPTOR.message_types_by_name['SharedFolderUpdateTeamStatus']
_SHAREDFOLDERUPDATEV3RESPONSE = DESCRIPTOR.message_types_by_name['SharedFolderUpdateV3Response']
EncryptedDataKey = _reflection.GeneratedProtocolMessageType('EncryptedDataKey', (_message.Message,), {
  'DESCRIPTOR' : _ENCRYPTEDDATAKEY,
  '__module__' : 'folder_pb2'
  # @@protoc_insertion_point(class_scope:Folder.EncryptedDataKey)
  })
_sym_db.RegisterMessage(EncryptedDataKey)

SharedFolderRecordData = _reflection.GeneratedProtocolMessageType('SharedFolderRecordData', (_message.Message,), {
  'DESCRIPTOR' : _SHAREDFOLDERRECORDDATA,
  '__module__' : 'folder_pb2'
  # @@protoc_insertion_point(class_scope:Folder.SharedFolderRecordData)
  })
_sym_db.RegisterMessage(SharedFolderRecordData)

SharedFolderRecordDataList = _reflection.GeneratedProtocolMessageType('SharedFolderRecordDataList', (_message.Message,), {
  'DESCRIPTOR' : _SHAREDFOLDERRECORDDATALIST,
  '__module__' : 'folder_pb2'
  # @@protoc_insertion_point(class_scope:Folder.SharedFolderRecordDataList)
  })
_sym_db.RegisterMessage(SharedFolderRecordDataList)

SharedFolderRecordFix = _reflection.GeneratedProtocolMessageType('SharedFolderRecordFix', (_message.Message,), {
  'DESCRIPTOR' : _SHAREDFOLDERRECORDFIX,
  '__module__' : 'folder_pb2'
  # @@protoc_insertion_point(class_scope:Folder.SharedFolderRecordFix)
  })
_sym_db.RegisterMessage(SharedFolderRecordFix)

SharedFolderRecordFixList = _reflection.GeneratedProtocolMessageType('SharedFolderRecordFixList', (_message.Message,), {
  'DESCRIPTOR' : _SHAREDFOLDERRECORDFIXLIST,
  '__module__' : 'folder_pb2'
  # @@protoc_insertion_point(class_scope:Folder.SharedFolderRecordFixList)
  })
_sym_db.RegisterMessage(SharedFolderRecordFixList)

RecordRequest = _reflection.GeneratedProtocolMessageType('RecordRequest', (_message.Message,), {
  'DESCRIPTOR' : _RECORDREQUEST,
  '__module__' : 'folder_pb2'
  # @@protoc_insertion_point(class_scope:Folder.RecordRequest)
  })
_sym_db.RegisterMessage(RecordRequest)

RecordResponse = _reflection.GeneratedProtocolMessageType('RecordResponse', (_message.Message,), {
  'DESCRIPTOR' : _RECORDRESPONSE,
  '__module__' : 'folder_pb2'
  # @@protoc_insertion_point(class_scope:Folder.RecordResponse)
  })
_sym_db.RegisterMessage(RecordResponse)

SharedFolderFields = _reflection.GeneratedProtocolMessageType('SharedFolderFields', (_message.Message,), {
  'DESCRIPTOR' : _SHAREDFOLDERFIELDS,
  '__module__' : 'folder_pb2'
  # @@protoc_insertion_point(class_scope:Folder.SharedFolderFields)
  })
_sym_db.RegisterMessage(SharedFolderFields)

SharedFolderFolderFields = _reflection.GeneratedProtocolMessageType('SharedFolderFolderFields', (_message.Message,), {
  'DESCRIPTOR' : _SHAREDFOLDERFOLDERFIELDS,
  '__module__' : 'folder_pb2'
  # @@protoc_insertion_point(class_scope:Folder.SharedFolderFolderFields)
  })
_sym_db.RegisterMessage(SharedFolderFolderFields)

FolderRequest = _reflection.GeneratedProtocolMessageType('FolderRequest', (_message.Message,), {
  'DESCRIPTOR' : _FOLDERREQUEST,
  '__module__' : 'folder_pb2'
  # @@protoc_insertion_point(class_scope:Folder.FolderRequest)
  })
_sym_db.RegisterMessage(FolderRequest)

FolderResponse = _reflection.GeneratedProtocolMessageType('FolderResponse', (_message.Message,), {
  'DESCRIPTOR' : _FOLDERRESPONSE,
  '__module__' : 'folder_pb2'
  # @@protoc_insertion_point(class_scope:Folder.FolderResponse)
  })
_sym_db.RegisterMessage(FolderResponse)

ImportFolderRecordRequest = _reflection.GeneratedProtocolMessageType('ImportFolderRecordRequest', (_message.Message,), {
  'DESCRIPTOR' : _IMPORTFOLDERRECORDREQUEST,
  '__module__' : 'folder_pb2'
  # @@protoc_insertion_point(class_scope:Folder.ImportFolderRecordRequest)
  })
_sym_db.RegisterMessage(ImportFolderRecordRequest)

ImportFolderRecordResponse = _reflection.GeneratedProtocolMessageType('ImportFolderRecordResponse', (_message.Message,), {
  'DESCRIPTOR' : _IMPORTFOLDERRECORDRESPONSE,
  '__module__' : 'folder_pb2'
  # @@protoc_insertion_point(class_scope:Folder.ImportFolderRecordResponse)
  })
_sym_db.RegisterMessage(ImportFolderRecordResponse)

SharedFolderUpdateRecord = _reflection.GeneratedProtocolMessageType('SharedFolderUpdateRecord', (_message.Message,), {
  'DESCRIPTOR' : _SHAREDFOLDERUPDATERECORD,
  '__module__' : 'folder_pb2'
  # @@protoc_insertion_point(class_scope:Folder.SharedFolderUpdateRecord)
  })
_sym_db.RegisterMessage(SharedFolderUpdateRecord)

SharedFolderUpdateUser = _reflection.GeneratedProtocolMessageType('SharedFolderUpdateUser', (_message.Message,), {
  'DESCRIPTOR' : _SHAREDFOLDERUPDATEUSER,
  '__module__' : 'folder_pb2'
  # @@protoc_insertion_point(class_scope:Folder.SharedFolderUpdateUser)
  })
_sym_db.RegisterMessage(SharedFolderUpdateUser)

SharedFolderUpdateTeam = _reflection.GeneratedProtocolMessageType('SharedFolderUpdateTeam', (_message.Message,), {
  'DESCRIPTOR' : _SHAREDFOLDERUPDATETEAM,
  '__module__' : 'folder_pb2'
  # @@protoc_insertion_point(class_scope:Folder.SharedFolderUpdateTeam)
  })
_sym_db.RegisterMessage(SharedFolderUpdateTeam)

SharedFolderUpdateV3Request = _reflection.GeneratedProtocolMessageType('SharedFolderUpdateV3Request', (_message.Message,), {
  'DESCRIPTOR' : _SHAREDFOLDERUPDATEV3REQUEST,
  '__module__' : 'folder_pb2'
  # @@protoc_insertion_point(class_scope:Folder.SharedFolderUpdateV3Request)
  })
_sym_db.RegisterMessage(SharedFolderUpdateV3Request)

SharedFolderUpdateRecordStatus = _reflection.GeneratedProtocolMessageType('SharedFolderUpdateRecordStatus', (_message.Message,), {
  'DESCRIPTOR' : _SHAREDFOLDERUPDATERECORDSTATUS,
  '__module__' : 'folder_pb2'
  # @@protoc_insertion_point(class_scope:Folder.SharedFolderUpdateRecordStatus)
  })
_sym_db.RegisterMessage(SharedFolderUpdateRecordStatus)

SharedFolderUpdateUserStatus = _reflection.GeneratedProtocolMessageType('SharedFolderUpdateUserStatus', (_message.Message,), {
  'DESCRIPTOR' : _SHAREDFOLDERUPDATEUSERSTATUS,
  '__module__' : 'folder_pb2'
  # @@protoc_insertion_point(class_scope:Folder.SharedFolderUpdateUserStatus)
  })
_sym_db.RegisterMessage(SharedFolderUpdateUserStatus)

SharedFolderUpdateTeamStatus = _reflection.GeneratedProtocolMessageType('SharedFolderUpdateTeamStatus', (_message.Message,), {
  'DESCRIPTOR' : _SHAREDFOLDERUPDATETEAMSTATUS,
  '__module__' : 'folder_pb2'
  # @@protoc_insertion_point(class_scope:Folder.SharedFolderUpdateTeamStatus)
  })
_sym_db.RegisterMessage(SharedFolderUpdateTeamStatus)

SharedFolderUpdateV3Response = _reflection.GeneratedProtocolMessageType('SharedFolderUpdateV3Response', (_message.Message,), {
  'DESCRIPTOR' : _SHAREDFOLDERUPDATEV3RESPONSE,
  '__module__' : 'folder_pb2'
  # @@protoc_insertion_point(class_scope:Folder.SharedFolderUpdateV3Response)
  })
_sym_db.RegisterMessage(SharedFolderUpdateV3Response)

if _descriptor._USE_C_DESCRIPTORS == False:

  DESCRIPTOR._options = None
  DESCRIPTOR._serialized_options = b'\n\030com.keepersecurity.protoB\006Folder'
  _RECORDTYPE._serialized_start=4082
  _RECORDTYPE._serialized_end=4108
  _FOLDERTYPE._serialized_start=4110
  _FOLDERTYPE._serialized_end=4204
  _ENCRYPTEDKEYTYPE._serialized_start=4206
  _ENCRYPTEDKEYTYPE._serialized_end=4292
  _SETBOOLEANVALUE._serialized_start=4294
  _SETBOOLEANVALUE._serialized_end=4371
  _ENCRYPTEDDATAKEY._serialized_start=24
  _ENCRYPTEDDATAKEY._serialized_end=116
  _SHAREDFOLDERRECORDDATA._serialized_start=119
  _SHAREDFOLDERRECORDDATA._serialized_end=249
  _SHAREDFOLDERRECORDDATALIST._serialized_start=251
  _SHAREDFOLDERRECORDDATALIST._serialized_end=343
  _SHAREDFOLDERRECORDFIX._serialized_start=345
  _SHAREDFOLDERRECORDFIX._serialized_end=440
  _SHAREDFOLDERRECORDFIXLIST._serialized_start=442
  _SHAREDFOLDERRECORDFIXLIST._serialized_end=531
  _RECORDREQUEST._serialized_start=534
  _RECORDREQUEST._serialized_end=824
  _RECORDRESPONSE._serialized_start=826
  _RECORDRESPONSE._serialized_end=895
  _SHAREDFOLDERFIELDS._serialized_start=898
  _SHAREDFOLDERFIELDS._serialized_end=1026
  _SHAREDFOLDERFOLDERFIELDS._serialized_start=1028
  _SHAREDFOLDERFOLDERFIELDS._serialized_end=1079
  _FOLDERREQUEST._serialized_start=1082
  _FOLDERREQUEST._serialized_end=1353
  _FOLDERRESPONSE._serialized_start=1355
  _FOLDERRESPONSE._serialized_end=1424
  _IMPORTFOLDERRECORDREQUEST._serialized_start=1426
  _IMPORTFOLDERRECORDREQUEST._serialized_end=1545
  _IMPORTFOLDERRECORDRESPONSE._serialized_start=1547
  _IMPORTFOLDERRECORDRESPONSE._serialized_end=1671
  _SHAREDFOLDERUPDATERECORD._serialized_start=1674
  _SHAREDFOLDERUPDATERECORD._serialized_end=1912
  _SHAREDFOLDERUPDATEUSER._serialized_start=1915
  _SHAREDFOLDERUPDATEUSER._serialized_end=2096
  _SHAREDFOLDERUPDATETEAM._serialized_start=2099
  _SHAREDFOLDERUPDATETEAM._serialized_end=2229
  _SHAREDFOLDERUPDATEV3REQUEST._serialized_start=2232
  _SHAREDFOLDERUPDATEV3REQUEST._serialized_end=3142
  _SHAREDFOLDERUPDATERECORDSTATUS._serialized_start=3144
  _SHAREDFOLDERUPDATERECORDSTATUS._serialized_end=3211
  _SHAREDFOLDERUPDATEUSERSTATUS._serialized_start=3213
  _SHAREDFOLDERUPDATEUSERSTATUS._serialized_end=3277
  _SHAREDFOLDERUPDATETEAMSTATUS._serialized_start=3279
  _SHAREDFOLDERUPDATETEAMSTATUS._serialized_end=3342
  _SHAREDFOLDERUPDATEV3RESPONSE._serialized_start=3345
  _SHAREDFOLDERUPDATEV3RESPONSE._serialized_end=4080
# @@protoc_insertion_point(module_scope)
