# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: client.proto
"""Generated protocol buffer code."""
from google.protobuf.internal import enum_type_wrapper
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import message as _message
from google.protobuf import reflection as _reflection
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()




DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n\x0c\x63lient.proto\x12\x06Tokens\"u\n\x18\x42reachWatchUpdateRequest\x12\x42\n\x18\x62reachWatchRecordRequest\x18\x01 \x03(\x0b\x32 .Tokens.BreachWatchRecordRequest\x12\x15\n\rencryptedData\x18\x02 \x01(\x0c\"\x9c\x01\n\x18\x42reachWatchRecordRequest\x12\x11\n\trecordUid\x18\x01 \x01(\x0c\x12\x15\n\rencryptedData\x18\x02 \x01(\x0c\x12\x38\n\x13\x62reachWatchInfoType\x18\x03 \x01(\x0e\x32\x1b.Tokens.BreachWatchInfoType\x12\x1c\n\x14updateUserWhoScanned\x18\x04 \x01(\x08\"\x81\x01\n\x0f\x42reachWatchData\x12%\n\tpasswords\x18\x01 \x03(\x0b\x32\x12.Tokens.BWPassword\x12\"\n\x06\x65mails\x18\x02 \x03(\x0b\x32\x12.Tokens.BWPassword\x12#\n\x07\x64omains\x18\x03 \x03(\x0b\x32\x12.Tokens.BWPassword\"]\n\nBWPassword\x12\r\n\x05value\x18\x01 \x01(\t\x12\x10\n\x08resolved\x18\x02 \x01(\x04\x12 \n\x06status\x18\x03 \x01(\x0e\x32\x10.Tokens.BWStatus\x12\x0c\n\x04\x65uid\x18\x04 \x01(\x0c*9\n\x13\x42reachWatchInfoType\x12\n\n\x06RECORD\x10\x00\x12\x16\n\x12\x41LTERNATE_PASSWORD\x10\x01*E\n\x08\x42WStatus\x12\x08\n\x04GOOD\x10\x00\x12\x0b\n\x07\x43HANGED\x10\x01\x12\x08\n\x04WEAK\x10\x02\x12\x0c\n\x08\x42REACHED\x10\x03\x12\n\n\x06IGNORE\x10\x04\x42\"\n\x18\x63om.keepersecurity.protoB\x06\x43lientb\x06proto3')

_BREACHWATCHINFOTYPE = DESCRIPTOR.enum_types_by_name['BreachWatchInfoType']
BreachWatchInfoType = enum_type_wrapper.EnumTypeWrapper(_BREACHWATCHINFOTYPE)
_BWSTATUS = DESCRIPTOR.enum_types_by_name['BWStatus']
BWStatus = enum_type_wrapper.EnumTypeWrapper(_BWSTATUS)
RECORD = 0
ALTERNATE_PASSWORD = 1
GOOD = 0
CHANGED = 1
WEAK = 2
BREACHED = 3
IGNORE = 4


_BREACHWATCHUPDATEREQUEST = DESCRIPTOR.message_types_by_name['BreachWatchUpdateRequest']
_BREACHWATCHRECORDREQUEST = DESCRIPTOR.message_types_by_name['BreachWatchRecordRequest']
_BREACHWATCHDATA = DESCRIPTOR.message_types_by_name['BreachWatchData']
_BWPASSWORD = DESCRIPTOR.message_types_by_name['BWPassword']
BreachWatchUpdateRequest = _reflection.GeneratedProtocolMessageType('BreachWatchUpdateRequest', (_message.Message,), {
  'DESCRIPTOR' : _BREACHWATCHUPDATEREQUEST,
  '__module__' : 'client_pb2'
  # @@protoc_insertion_point(class_scope:Tokens.BreachWatchUpdateRequest)
  })
_sym_db.RegisterMessage(BreachWatchUpdateRequest)

BreachWatchRecordRequest = _reflection.GeneratedProtocolMessageType('BreachWatchRecordRequest', (_message.Message,), {
  'DESCRIPTOR' : _BREACHWATCHRECORDREQUEST,
  '__module__' : 'client_pb2'
  # @@protoc_insertion_point(class_scope:Tokens.BreachWatchRecordRequest)
  })
_sym_db.RegisterMessage(BreachWatchRecordRequest)

BreachWatchData = _reflection.GeneratedProtocolMessageType('BreachWatchData', (_message.Message,), {
  'DESCRIPTOR' : _BREACHWATCHDATA,
  '__module__' : 'client_pb2'
  # @@protoc_insertion_point(class_scope:Tokens.BreachWatchData)
  })
_sym_db.RegisterMessage(BreachWatchData)

BWPassword = _reflection.GeneratedProtocolMessageType('BWPassword', (_message.Message,), {
  'DESCRIPTOR' : _BWPASSWORD,
  '__module__' : 'client_pb2'
  # @@protoc_insertion_point(class_scope:Tokens.BWPassword)
  })
_sym_db.RegisterMessage(BWPassword)

if _descriptor._USE_C_DESCRIPTORS == False:

  DESCRIPTOR._options = None
  DESCRIPTOR._serialized_options = b'\n\030com.keepersecurity.protoB\006Client'
  _BREACHWATCHINFOTYPE._serialized_start=529
  _BREACHWATCHINFOTYPE._serialized_end=586
  _BWSTATUS._serialized_start=588
  _BWSTATUS._serialized_end=657
  _BREACHWATCHUPDATEREQUEST._serialized_start=24
  _BREACHWATCHUPDATEREQUEST._serialized_end=141
  _BREACHWATCHRECORDREQUEST._serialized_start=144
  _BREACHWATCHRECORDREQUEST._serialized_end=300
  _BREACHWATCHDATA._serialized_start=303
  _BREACHWATCHDATA._serialized_end=432
  _BWPASSWORD._serialized_start=434
  _BWPASSWORD._serialized_end=527
# @@protoc_insertion_point(module_scope)
