# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: SyncDown.proto
"""Generated protocol buffer code."""
from google.protobuf.internal import enum_type_wrapper
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import message as _message
from google.protobuf import reflection as _reflection
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from . import record_pb2 as record__pb2
from . import breachwatch_pb2 as breachwatch__pb2
from . import APIRequest_pb2 as APIRequest__pb2


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n\x0eSyncDown.proto\x12\x05Vault\x1a\x0crecord.proto\x1a\x11\x62reachwatch.proto\x1a\x10\x41PIRequest.proto\",\n\x0fSyncDownRequest\x12\x19\n\x11\x63ontinuationToken\x18\x01 \x01(\x0c\"\x89\x10\n\x10SyncDownResponse\x12\x19\n\x11\x63ontinuationToken\x18\x01 \x01(\x0c\x12\x0f\n\x07hasMore\x18\x02 \x01(\x08\x12\'\n\x0b\x63\x61\x63heStatus\x18\x03 \x01(\x0e\x32\x12.Vault.CacheStatus\x12&\n\x0buserFolders\x18\x04 \x03(\x0b\x32\x11.Vault.UserFolder\x12*\n\rsharedFolders\x18\x05 \x03(\x0b\x32\x13.Vault.SharedFolder\x12>\n\x17userFolderSharedFolders\x18\x06 \x03(\x0b\x32\x1d.Vault.UserFolderSharedFolder\x12\x36\n\x13sharedFolderFolders\x18\x07 \x03(\x0b\x32\x19.Vault.SharedFolderFolder\x12\x1e\n\x07records\x18\x08 \x03(\x0b\x32\r.Vault.Record\x12-\n\x0erecordMetaData\x18\t \x03(\x0b\x32\x15.Vault.RecordMetaData\x12+\n\rnonSharedData\x18\n \x03(\x0b\x32\x14.Vault.NonSharedData\x12&\n\x0brecordLinks\x18\x0b \x03(\x0b\x32\x11.Vault.RecordLink\x12\x32\n\x11userFolderRecords\x18\x0c \x03(\x0b\x32\x17.Vault.UserFolderRecord\x12\x36\n\x13sharedFolderRecords\x18\r \x03(\x0b\x32\x19.Vault.SharedFolderRecord\x12\x42\n\x19sharedFolderFolderRecords\x18\x0e \x03(\x0b\x32\x1f.Vault.SharedFolderFolderRecord\x12\x32\n\x11sharedFolderUsers\x18\x0f \x03(\x0b\x32\x17.Vault.SharedFolderUser\x12\x32\n\x11sharedFolderTeams\x18\x10 \x03(\x0b\x32\x17.Vault.SharedFolderTeam\x12\x1a\n\x12recordAddAuditData\x18\x11 \x03(\x0c\x12\x1a\n\x05teams\x18\x12 \x03(\x0b\x32\x0b.Vault.Team\x12,\n\x0esharingChanges\x18\x13 \x03(\x0b\x32\x14.Vault.SharingChange\x12\x1f\n\x07profile\x18\x14 \x01(\x0b\x32\x0e.Vault.Profile\x12%\n\nprofilePic\x18\x15 \x01(\x0b\x32\x11.Vault.ProfilePic\x12\x34\n\x12pendingTeamMembers\x18\x16 \x03(\x0b\x32\x18.Vault.PendingTeamMember\x12\x34\n\x12\x62reachWatchRecords\x18\x17 \x03(\x0b\x32\x18.Vault.BreachWatchRecord\x12\"\n\tuserAuths\x18\x18 \x03(\x0b\x32\x0f.Vault.UserAuth\x12?\n\x17\x62reachWatchSecurityData\x18\x19 \x03(\x0b\x32\x1e.Vault.BreachWatchSecurityData\x12/\n\x0freusedPasswords\x18\x1a \x01(\x0b\x32\x16.Vault.ReusedPasswords\x12\x1a\n\x12removedUserFolders\x18\x1b \x03(\x0c\x12\x1c\n\x14removedSharedFolders\x18\x1c \x03(\x0c\x12\x45\n\x1eremovedUserFolderSharedFolders\x18\x1d \x03(\x0b\x32\x1d.Vault.UserFolderSharedFolder\x12=\n\x1aremovedSharedFolderFolders\x18\x1e \x03(\x0b\x32\x19.Vault.SharedFolderFolder\x12\x16\n\x0eremovedRecords\x18\x1f \x03(\x0c\x12-\n\x12removedRecordLinks\x18  \x03(\x0b\x32\x11.Vault.RecordLink\x12\x39\n\x18removedUserFolderRecords\x18! \x03(\x0b\x32\x17.Vault.UserFolderRecord\x12=\n\x1aremovedSharedFolderRecords\x18\" \x03(\x0b\x32\x19.Vault.SharedFolderRecord\x12I\n removedSharedFolderFolderRecords\x18# \x03(\x0b\x32\x1f.Vault.SharedFolderFolderRecord\x12\x39\n\x18removedSharedFolderUsers\x18$ \x03(\x0b\x32\x17.Vault.SharedFolderUser\x12\x39\n\x18removedSharedFolderTeams\x18% \x03(\x0b\x32\x17.Vault.SharedFolderTeam\x12\x14\n\x0cremovedTeams\x18& \x03(\x0c\x12&\n\x0cksmAppShares\x18\' \x03(\x0b\x32\x10.Vault.KsmChange\x12\'\n\rksmAppClients\x18( \x03(\x0b\x32\x10.Vault.KsmChange\x12\x30\n\x10shareInvitations\x18) \x03(\x0b\x32\x16.Vault.ShareInvitation\x12+\n\x0b\x64iagnostics\x18* \x01(\x0b\x32\x16.Vault.SyncDiagnostics\x12.\n\x0frecordRotations\x18+ \x03(\x0b\x32\x15.Vault.RecordRotation\x12\x1a\n\x05users\x18, \x03(\x0b\x32\x0b.Vault.User\x12\x14\n\x0cremovedUsers\x18- \x03(\x0c\"\x92\x01\n\nUserFolder\x12\x11\n\tfolderUid\x18\x01 \x01(\x0c\x12\x11\n\tparentUid\x18\x02 \x01(\x0c\x12\x15\n\ruserFolderKey\x18\x03 \x01(\x0c\x12\'\n\x07keyType\x18\x04 \x01(\x0e\x32\x16.Records.RecordKeyType\x12\x10\n\x08revision\x18\x05 \x01(\x03\x12\x0c\n\x04\x64\x61ta\x18\x06 \x01(\x0c\"\xc7\x02\n\x0cSharedFolder\x12\x17\n\x0fsharedFolderUid\x18\x01 \x01(\x0c\x12\x10\n\x08revision\x18\x02 \x01(\x03\x12\x17\n\x0fsharedFolderKey\x18\x03 \x01(\x0c\x12\'\n\x07keyType\x18\x04 \x01(\x0e\x32\x16.Records.RecordKeyType\x12\x0c\n\x04\x64\x61ta\x18\x05 \x01(\x0c\x12\x1c\n\x14\x64\x65\x66\x61ultManageRecords\x18\x06 \x01(\x08\x12\x1a\n\x12\x64\x65\x66\x61ultManageUsers\x18\x07 \x01(\x08\x12\x16\n\x0e\x64\x65\x66\x61ultCanEdit\x18\x08 \x01(\x08\x12\x19\n\x11\x64\x65\x66\x61ultCanReshare\x18\t \x01(\x08\x12\'\n\x0b\x63\x61\x63heStatus\x18\n \x01(\x0e\x32\x12.Vault.CacheStatus\x12\r\n\x05owner\x18\x0b \x01(\t\x12\x17\n\x0fownerAccountUid\x18\x0c \x01(\x0c\"V\n\x16UserFolderSharedFolder\x12\x11\n\tfolderUid\x18\x01 \x01(\x0c\x12\x17\n\x0fsharedFolderUid\x18\x02 \x01(\x0c\x12\x10\n\x08revision\x18\x03 \x01(\x03\"\xbb\x01\n\x12SharedFolderFolder\x12\x17\n\x0fsharedFolderUid\x18\x01 \x01(\x0c\x12\x11\n\tfolderUid\x18\x02 \x01(\x0c\x12\x11\n\tparentUid\x18\x03 \x01(\x0c\x12\x1d\n\x15sharedFolderFolderKey\x18\x04 \x01(\x0c\x12\'\n\x07keyType\x18\x05 \x01(\x0e\x32\x16.Records.RecordKeyType\x12\x10\n\x08revision\x18\x06 \x01(\x03\x12\x0c\n\x04\x64\x61ta\x18\x07 \x01(\x0c\"l\n\x0fSharedFolderKey\x12\x17\n\x0fsharedFolderUid\x18\x01 \x01(\x0c\x12\x17\n\x0fsharedFolderKey\x18\x02 \x01(\x0c\x12\'\n\x07keyType\x18\x03 \x01(\x0e\x32\x16.Records.RecordKeyType\"\x8e\x02\n\x04Team\x12\x0f\n\x07teamUid\x18\x01 \x01(\x0c\x12\x0c\n\x04name\x18\x02 \x01(\t\x12\x0f\n\x07teamKey\x18\x03 \x01(\x0c\x12+\n\x0bteamKeyType\x18\x04 \x01(\x0e\x32\x16.Records.RecordKeyType\x12\x16\n\x0eteamPrivateKey\x18\x05 \x01(\x0c\x12\x14\n\x0crestrictEdit\x18\x06 \x01(\x08\x12\x15\n\rrestrictShare\x18\x07 \x01(\x08\x12\x14\n\x0crestrictView\x18\x08 \x01(\x08\x12\x1c\n\x14removedSharedFolders\x18\t \x03(\x0c\x12\x30\n\x10sharedFolderKeys\x18\n \x03(\x0b\x32\x16.Vault.SharedFolderKey\"\xbf\x01\n\x06Record\x12\x11\n\trecordUid\x18\x01 \x01(\x0c\x12\x10\n\x08revision\x18\x02 \x01(\x03\x12\x0f\n\x07version\x18\x03 \x01(\x05\x12\x0e\n\x06shared\x18\x04 \x01(\x08\x12\x1a\n\x12\x63lientModifiedTime\x18\x05 \x01(\x03\x12\x0c\n\x04\x64\x61ta\x18\x06 \x01(\x0c\x12\r\n\x05\x65xtra\x18\x07 \x01(\x0c\x12\r\n\x05udata\x18\x08 \x01(\t\x12\x10\n\x08\x66ileSize\x18\t \x01(\x03\x12\x15\n\rthumbnailSize\x18\n \x01(\x03\"b\n\nRecordLink\x12\x17\n\x0fparentRecordUid\x18\x01 \x01(\x0c\x12\x16\n\x0e\x63hildRecordUid\x18\x02 \x01(\x0c\x12\x11\n\trecordKey\x18\x03 \x01(\x0c\x12\x10\n\x08revision\x18\x04 \x01(\x03\"J\n\x10UserFolderRecord\x12\x11\n\tfolderUid\x18\x01 \x01(\x0c\x12\x11\n\trecordUid\x18\x02 \x01(\x0c\x12\x10\n\x08revision\x18\x03 \x01(\x03\"Y\n\x18SharedFolderFolderRecord\x12\x17\n\x0fsharedFolderUid\x18\x01 \x01(\x0c\x12\x11\n\tfolderUid\x18\x02 \x01(\x0c\x12\x11\n\trecordUid\x18\x03 \x01(\x0c\"0\n\rNonSharedData\x12\x11\n\trecordUid\x18\x01 \x01(\x0c\x12\x0c\n\x04\x64\x61ta\x18\x02 \x01(\x0c\"\xc4\x01\n\x0eRecordMetaData\x12\x11\n\trecordUid\x18\x01 \x01(\x0c\x12\r\n\x05owner\x18\x02 \x01(\x08\x12\x11\n\trecordKey\x18\x03 \x01(\x0c\x12-\n\rrecordKeyType\x18\x04 \x01(\x0e\x32\x16.Records.RecordKeyType\x12\x10\n\x08\x63\x61nShare\x18\x05 \x01(\x08\x12\x0f\n\x07\x63\x61nEdit\x18\x06 \x01(\x08\x12\x17\n\x0fownerAccountUid\x18\x07 \x01(\x0c\x12\x12\n\nexpiration\x18\x08 \x01(\x03\"2\n\rSharingChange\x12\x11\n\trecordUid\x18\x01 \x01(\x0c\x12\x0e\n\x06shared\x18\x02 \x01(\x08\">\n\x07Profile\x12\x0c\n\x04\x64\x61ta\x18\x01 \x01(\x0c\x12\x13\n\x0bprofileName\x18\x02 \x01(\t\x12\x10\n\x08revision\x18\x03 \x01(\x03\"+\n\nProfilePic\x12\x0b\n\x03url\x18\x01 \x01(\t\x12\x10\n\x08revision\x18\x02 \x01(\x03\"V\n\x11PendingTeamMember\x12\x18\n\x10\x65nterpriseUserId\x18\x01 \x01(\x03\x12\x15\n\ruserPublicKey\x18\x02 \x01(\x0c\x12\x10\n\x08teamUids\x18\x03 \x03(\x0c\"\x89\x01\n\x11\x42reachWatchRecord\x12\x11\n\trecordUid\x18\x01 \x01(\x0c\x12\x0c\n\x04\x64\x61ta\x18\x02 \x01(\x0c\x12.\n\x04type\x18\x03 \x01(\x0e\x32 .BreachWatch.BreachWatchInfoType\x12\x11\n\tscannedBy\x18\x04 \x01(\t\x12\x10\n\x08revision\x18\x05 \x01(\x03\"\xb4\x01\n\x08UserAuth\x12\x0b\n\x03uid\x18\x01 \x01(\x0c\x12,\n\tloginType\x18\x02 \x01(\x0e\x32\x19.Authentication.LoginType\x12\x0f\n\x07\x64\x65leted\x18\x03 \x01(\x08\x12\x12\n\niterations\x18\x04 \x01(\x05\x12\x0c\n\x04salt\x18\x05 \x01(\x0c\x12\x1a\n\x12\x65ncryptedClientKey\x18\x06 \x01(\x0c\x12\x10\n\x08revision\x18\x07 \x01(\x03\x12\x0c\n\x04name\x18\x08 \x01(\t\">\n\x17\x42reachWatchSecurityData\x12\x11\n\trecordUid\x18\x01 \x01(\x0c\x12\x10\n\x08revision\x18\x02 \x01(\x03\"2\n\x0fReusedPasswords\x12\r\n\x05\x63ount\x18\x01 \x01(\x05\x12\x10\n\x08revision\x18\x02 \x01(\x03\"\xa3\x01\n\x12SharedFolderRecord\x12\x17\n\x0fsharedFolderUid\x18\x01 \x01(\x0c\x12\x11\n\trecordUid\x18\x02 \x01(\x0c\x12\x11\n\trecordKey\x18\x03 \x01(\x0c\x12\x10\n\x08\x63\x61nShare\x18\x04 \x01(\x08\x12\x0f\n\x07\x63\x61nEdit\x18\x05 \x01(\x08\x12\x17\n\x0fownerAccountUid\x18\x06 \x01(\x0c\x12\x12\n\nexpiration\x18\x07 \x01(\x03\"\x91\x01\n\x10SharedFolderUser\x12\x17\n\x0fsharedFolderUid\x18\x01 \x01(\x0c\x12\x10\n\x08username\x18\x02 \x01(\t\x12\x15\n\rmanageRecords\x18\x03 \x01(\x08\x12\x13\n\x0bmanageUsers\x18\x04 \x01(\x08\x12\x12\n\naccountUid\x18\x05 \x01(\x0c\x12\x12\n\nexpiration\x18\x06 \x01(\x03\"\x8a\x01\n\x10SharedFolderTeam\x12\x17\n\x0fsharedFolderUid\x18\x01 \x01(\x0c\x12\x0f\n\x07teamUid\x18\x02 \x01(\x0c\x12\x0c\n\x04name\x18\x03 \x01(\t\x12\x15\n\rmanageRecords\x18\x04 \x01(\x08\x12\x13\n\x0bmanageUsers\x18\x05 \x01(\x08\x12\x12\n\nexpiration\x18\x06 \x01(\x03\"D\n\tKsmChange\x12\x14\n\x0c\x61ppRecordUid\x18\x01 \x01(\x0c\x12\x10\n\x08\x64\x65tailId\x18\x02 \x01(\x0c\x12\x0f\n\x07removed\x18\x03 \x01(\x08\"#\n\x0fShareInvitation\x12\x10\n\x08username\x18\x01 \x01(\t\",\n\x04User\x12\x12\n\naccountUid\x18\x01 \x01(\x0c\x12\x10\n\x08username\x18\x02 \x01(\t\"{\n\x0fSyncDiagnostics\x12\x19\n\x11\x63ontinuationToken\x18\x01 \x01(\x0c\x12\x0e\n\x06userId\x18\x02 \x01(\x05\x12\x18\n\x10\x65nterpriseUserId\x18\x03 \x01(\x03\x12\x10\n\x08syncedTo\x18\x04 \x01(\x03\x12\x11\n\tsyncingTo\x18\x05 \x01(\x03\"\x9d\x01\n\x0eRecordRotation\x12\x11\n\trecordUid\x18\x01 \x01(\x0c\x12\x10\n\x08revision\x18\x02 \x01(\x03\x12\x18\n\x10\x63onfigurationUid\x18\x03 \x01(\x0c\x12\x10\n\x08schedule\x18\x04 \x01(\t\x12\x15\n\rpwdComplexity\x18\x05 \x01(\x0c\x12\x10\n\x08\x64isabled\x18\x06 \x01(\x08\x12\x11\n\tparentUid\x18\x07 \x01(\x0c\"3\n\x1d\x42reachWatchGetSyncDataRequest\x12\x12\n\nrecordUids\x18\x01 \x03(\x0c\"\x97\x01\n\x1e\x42reachWatchGetSyncDataResponse\x12\x34\n\x12\x62reachWatchRecords\x18\x01 \x03(\x0b\x32\x18.Vault.BreachWatchRecord\x12?\n\x17\x62reachWatchSecurityData\x18\x02 \x03(\x0b\x32\x1e.Vault.BreachWatchSecurityData*\"\n\x0b\x43\x61\x63heStatus\x12\x08\n\x04KEEP\x10\x00\x12\t\n\x05\x43LEAR\x10\x01\x42!\n\x18\x63om.keepersecurity.protoB\x05Vaultb\x06proto3')

_CACHESTATUS = DESCRIPTOR.enum_types_by_name['CacheStatus']
CacheStatus = enum_type_wrapper.EnumTypeWrapper(_CACHESTATUS)
KEEP = 0
CLEAR = 1


_SYNCDOWNREQUEST = DESCRIPTOR.message_types_by_name['SyncDownRequest']
_SYNCDOWNRESPONSE = DESCRIPTOR.message_types_by_name['SyncDownResponse']
_USERFOLDER = DESCRIPTOR.message_types_by_name['UserFolder']
_SHAREDFOLDER = DESCRIPTOR.message_types_by_name['SharedFolder']
_USERFOLDERSHAREDFOLDER = DESCRIPTOR.message_types_by_name['UserFolderSharedFolder']
_SHAREDFOLDERFOLDER = DESCRIPTOR.message_types_by_name['SharedFolderFolder']
_SHAREDFOLDERKEY = DESCRIPTOR.message_types_by_name['SharedFolderKey']
_TEAM = DESCRIPTOR.message_types_by_name['Team']
_RECORD = DESCRIPTOR.message_types_by_name['Record']
_RECORDLINK = DESCRIPTOR.message_types_by_name['RecordLink']
_USERFOLDERRECORD = DESCRIPTOR.message_types_by_name['UserFolderRecord']
_SHAREDFOLDERFOLDERRECORD = DESCRIPTOR.message_types_by_name['SharedFolderFolderRecord']
_NONSHAREDDATA = DESCRIPTOR.message_types_by_name['NonSharedData']
_RECORDMETADATA = DESCRIPTOR.message_types_by_name['RecordMetaData']
_SHARINGCHANGE = DESCRIPTOR.message_types_by_name['SharingChange']
_PROFILE = DESCRIPTOR.message_types_by_name['Profile']
_PROFILEPIC = DESCRIPTOR.message_types_by_name['ProfilePic']
_PENDINGTEAMMEMBER = DESCRIPTOR.message_types_by_name['PendingTeamMember']
_BREACHWATCHRECORD = DESCRIPTOR.message_types_by_name['BreachWatchRecord']
_USERAUTH = DESCRIPTOR.message_types_by_name['UserAuth']
_BREACHWATCHSECURITYDATA = DESCRIPTOR.message_types_by_name['BreachWatchSecurityData']
_REUSEDPASSWORDS = DESCRIPTOR.message_types_by_name['ReusedPasswords']
_SHAREDFOLDERRECORD = DESCRIPTOR.message_types_by_name['SharedFolderRecord']
_SHAREDFOLDERUSER = DESCRIPTOR.message_types_by_name['SharedFolderUser']
_SHAREDFOLDERTEAM = DESCRIPTOR.message_types_by_name['SharedFolderTeam']
_KSMCHANGE = DESCRIPTOR.message_types_by_name['KsmChange']
_SHAREINVITATION = DESCRIPTOR.message_types_by_name['ShareInvitation']
_USER = DESCRIPTOR.message_types_by_name['User']
_SYNCDIAGNOSTICS = DESCRIPTOR.message_types_by_name['SyncDiagnostics']
_RECORDROTATION = DESCRIPTOR.message_types_by_name['RecordRotation']
_BREACHWATCHGETSYNCDATAREQUEST = DESCRIPTOR.message_types_by_name['BreachWatchGetSyncDataRequest']
_BREACHWATCHGETSYNCDATARESPONSE = DESCRIPTOR.message_types_by_name['BreachWatchGetSyncDataResponse']
SyncDownRequest = _reflection.GeneratedProtocolMessageType('SyncDownRequest', (_message.Message,), {
  'DESCRIPTOR' : _SYNCDOWNREQUEST,
  '__module__' : 'SyncDown_pb2'
  # @@protoc_insertion_point(class_scope:Vault.SyncDownRequest)
  })
_sym_db.RegisterMessage(SyncDownRequest)

SyncDownResponse = _reflection.GeneratedProtocolMessageType('SyncDownResponse', (_message.Message,), {
  'DESCRIPTOR' : _SYNCDOWNRESPONSE,
  '__module__' : 'SyncDown_pb2'
  # @@protoc_insertion_point(class_scope:Vault.SyncDownResponse)
  })
_sym_db.RegisterMessage(SyncDownResponse)

UserFolder = _reflection.GeneratedProtocolMessageType('UserFolder', (_message.Message,), {
  'DESCRIPTOR' : _USERFOLDER,
  '__module__' : 'SyncDown_pb2'
  # @@protoc_insertion_point(class_scope:Vault.UserFolder)
  })
_sym_db.RegisterMessage(UserFolder)

SharedFolder = _reflection.GeneratedProtocolMessageType('SharedFolder', (_message.Message,), {
  'DESCRIPTOR' : _SHAREDFOLDER,
  '__module__' : 'SyncDown_pb2'
  # @@protoc_insertion_point(class_scope:Vault.SharedFolder)
  })
_sym_db.RegisterMessage(SharedFolder)

UserFolderSharedFolder = _reflection.GeneratedProtocolMessageType('UserFolderSharedFolder', (_message.Message,), {
  'DESCRIPTOR' : _USERFOLDERSHAREDFOLDER,
  '__module__' : 'SyncDown_pb2'
  # @@protoc_insertion_point(class_scope:Vault.UserFolderSharedFolder)
  })
_sym_db.RegisterMessage(UserFolderSharedFolder)

SharedFolderFolder = _reflection.GeneratedProtocolMessageType('SharedFolderFolder', (_message.Message,), {
  'DESCRIPTOR' : _SHAREDFOLDERFOLDER,
  '__module__' : 'SyncDown_pb2'
  # @@protoc_insertion_point(class_scope:Vault.SharedFolderFolder)
  })
_sym_db.RegisterMessage(SharedFolderFolder)

SharedFolderKey = _reflection.GeneratedProtocolMessageType('SharedFolderKey', (_message.Message,), {
  'DESCRIPTOR' : _SHAREDFOLDERKEY,
  '__module__' : 'SyncDown_pb2'
  # @@protoc_insertion_point(class_scope:Vault.SharedFolderKey)
  })
_sym_db.RegisterMessage(SharedFolderKey)

Team = _reflection.GeneratedProtocolMessageType('Team', (_message.Message,), {
  'DESCRIPTOR' : _TEAM,
  '__module__' : 'SyncDown_pb2'
  # @@protoc_insertion_point(class_scope:Vault.Team)
  })
_sym_db.RegisterMessage(Team)

Record = _reflection.GeneratedProtocolMessageType('Record', (_message.Message,), {
  'DESCRIPTOR' : _RECORD,
  '__module__' : 'SyncDown_pb2'
  # @@protoc_insertion_point(class_scope:Vault.Record)
  })
_sym_db.RegisterMessage(Record)

RecordLink = _reflection.GeneratedProtocolMessageType('RecordLink', (_message.Message,), {
  'DESCRIPTOR' : _RECORDLINK,
  '__module__' : 'SyncDown_pb2'
  # @@protoc_insertion_point(class_scope:Vault.RecordLink)
  })
_sym_db.RegisterMessage(RecordLink)

UserFolderRecord = _reflection.GeneratedProtocolMessageType('UserFolderRecord', (_message.Message,), {
  'DESCRIPTOR' : _USERFOLDERRECORD,
  '__module__' : 'SyncDown_pb2'
  # @@protoc_insertion_point(class_scope:Vault.UserFolderRecord)
  })
_sym_db.RegisterMessage(UserFolderRecord)

SharedFolderFolderRecord = _reflection.GeneratedProtocolMessageType('SharedFolderFolderRecord', (_message.Message,), {
  'DESCRIPTOR' : _SHAREDFOLDERFOLDERRECORD,
  '__module__' : 'SyncDown_pb2'
  # @@protoc_insertion_point(class_scope:Vault.SharedFolderFolderRecord)
  })
_sym_db.RegisterMessage(SharedFolderFolderRecord)

NonSharedData = _reflection.GeneratedProtocolMessageType('NonSharedData', (_message.Message,), {
  'DESCRIPTOR' : _NONSHAREDDATA,
  '__module__' : 'SyncDown_pb2'
  # @@protoc_insertion_point(class_scope:Vault.NonSharedData)
  })
_sym_db.RegisterMessage(NonSharedData)

RecordMetaData = _reflection.GeneratedProtocolMessageType('RecordMetaData', (_message.Message,), {
  'DESCRIPTOR' : _RECORDMETADATA,
  '__module__' : 'SyncDown_pb2'
  # @@protoc_insertion_point(class_scope:Vault.RecordMetaData)
  })
_sym_db.RegisterMessage(RecordMetaData)

SharingChange = _reflection.GeneratedProtocolMessageType('SharingChange', (_message.Message,), {
  'DESCRIPTOR' : _SHARINGCHANGE,
  '__module__' : 'SyncDown_pb2'
  # @@protoc_insertion_point(class_scope:Vault.SharingChange)
  })
_sym_db.RegisterMessage(SharingChange)

Profile = _reflection.GeneratedProtocolMessageType('Profile', (_message.Message,), {
  'DESCRIPTOR' : _PROFILE,
  '__module__' : 'SyncDown_pb2'
  # @@protoc_insertion_point(class_scope:Vault.Profile)
  })
_sym_db.RegisterMessage(Profile)

ProfilePic = _reflection.GeneratedProtocolMessageType('ProfilePic', (_message.Message,), {
  'DESCRIPTOR' : _PROFILEPIC,
  '__module__' : 'SyncDown_pb2'
  # @@protoc_insertion_point(class_scope:Vault.ProfilePic)
  })
_sym_db.RegisterMessage(ProfilePic)

PendingTeamMember = _reflection.GeneratedProtocolMessageType('PendingTeamMember', (_message.Message,), {
  'DESCRIPTOR' : _PENDINGTEAMMEMBER,
  '__module__' : 'SyncDown_pb2'
  # @@protoc_insertion_point(class_scope:Vault.PendingTeamMember)
  })
_sym_db.RegisterMessage(PendingTeamMember)

BreachWatchRecord = _reflection.GeneratedProtocolMessageType('BreachWatchRecord', (_message.Message,), {
  'DESCRIPTOR' : _BREACHWATCHRECORD,
  '__module__' : 'SyncDown_pb2'
  # @@protoc_insertion_point(class_scope:Vault.BreachWatchRecord)
  })
_sym_db.RegisterMessage(BreachWatchRecord)

UserAuth = _reflection.GeneratedProtocolMessageType('UserAuth', (_message.Message,), {
  'DESCRIPTOR' : _USERAUTH,
  '__module__' : 'SyncDown_pb2'
  # @@protoc_insertion_point(class_scope:Vault.UserAuth)
  })
_sym_db.RegisterMessage(UserAuth)

BreachWatchSecurityData = _reflection.GeneratedProtocolMessageType('BreachWatchSecurityData', (_message.Message,), {
  'DESCRIPTOR' : _BREACHWATCHSECURITYDATA,
  '__module__' : 'SyncDown_pb2'
  # @@protoc_insertion_point(class_scope:Vault.BreachWatchSecurityData)
  })
_sym_db.RegisterMessage(BreachWatchSecurityData)

ReusedPasswords = _reflection.GeneratedProtocolMessageType('ReusedPasswords', (_message.Message,), {
  'DESCRIPTOR' : _REUSEDPASSWORDS,
  '__module__' : 'SyncDown_pb2'
  # @@protoc_insertion_point(class_scope:Vault.ReusedPasswords)
  })
_sym_db.RegisterMessage(ReusedPasswords)

SharedFolderRecord = _reflection.GeneratedProtocolMessageType('SharedFolderRecord', (_message.Message,), {
  'DESCRIPTOR' : _SHAREDFOLDERRECORD,
  '__module__' : 'SyncDown_pb2'
  # @@protoc_insertion_point(class_scope:Vault.SharedFolderRecord)
  })
_sym_db.RegisterMessage(SharedFolderRecord)

SharedFolderUser = _reflection.GeneratedProtocolMessageType('SharedFolderUser', (_message.Message,), {
  'DESCRIPTOR' : _SHAREDFOLDERUSER,
  '__module__' : 'SyncDown_pb2'
  # @@protoc_insertion_point(class_scope:Vault.SharedFolderUser)
  })
_sym_db.RegisterMessage(SharedFolderUser)

SharedFolderTeam = _reflection.GeneratedProtocolMessageType('SharedFolderTeam', (_message.Message,), {
  'DESCRIPTOR' : _SHAREDFOLDERTEAM,
  '__module__' : 'SyncDown_pb2'
  # @@protoc_insertion_point(class_scope:Vault.SharedFolderTeam)
  })
_sym_db.RegisterMessage(SharedFolderTeam)

KsmChange = _reflection.GeneratedProtocolMessageType('KsmChange', (_message.Message,), {
  'DESCRIPTOR' : _KSMCHANGE,
  '__module__' : 'SyncDown_pb2'
  # @@protoc_insertion_point(class_scope:Vault.KsmChange)
  })
_sym_db.RegisterMessage(KsmChange)

ShareInvitation = _reflection.GeneratedProtocolMessageType('ShareInvitation', (_message.Message,), {
  'DESCRIPTOR' : _SHAREINVITATION,
  '__module__' : 'SyncDown_pb2'
  # @@protoc_insertion_point(class_scope:Vault.ShareInvitation)
  })
_sym_db.RegisterMessage(ShareInvitation)

User = _reflection.GeneratedProtocolMessageType('User', (_message.Message,), {
  'DESCRIPTOR' : _USER,
  '__module__' : 'SyncDown_pb2'
  # @@protoc_insertion_point(class_scope:Vault.User)
  })
_sym_db.RegisterMessage(User)

SyncDiagnostics = _reflection.GeneratedProtocolMessageType('SyncDiagnostics', (_message.Message,), {
  'DESCRIPTOR' : _SYNCDIAGNOSTICS,
  '__module__' : 'SyncDown_pb2'
  # @@protoc_insertion_point(class_scope:Vault.SyncDiagnostics)
  })
_sym_db.RegisterMessage(SyncDiagnostics)

RecordRotation = _reflection.GeneratedProtocolMessageType('RecordRotation', (_message.Message,), {
  'DESCRIPTOR' : _RECORDROTATION,
  '__module__' : 'SyncDown_pb2'
  # @@protoc_insertion_point(class_scope:Vault.RecordRotation)
  })
_sym_db.RegisterMessage(RecordRotation)

BreachWatchGetSyncDataRequest = _reflection.GeneratedProtocolMessageType('BreachWatchGetSyncDataRequest', (_message.Message,), {
  'DESCRIPTOR' : _BREACHWATCHGETSYNCDATAREQUEST,
  '__module__' : 'SyncDown_pb2'
  # @@protoc_insertion_point(class_scope:Vault.BreachWatchGetSyncDataRequest)
  })
_sym_db.RegisterMessage(BreachWatchGetSyncDataRequest)

BreachWatchGetSyncDataResponse = _reflection.GeneratedProtocolMessageType('BreachWatchGetSyncDataResponse', (_message.Message,), {
  'DESCRIPTOR' : _BREACHWATCHGETSYNCDATARESPONSE,
  '__module__' : 'SyncDown_pb2'
  # @@protoc_insertion_point(class_scope:Vault.BreachWatchGetSyncDataResponse)
  })
_sym_db.RegisterMessage(BreachWatchGetSyncDataResponse)

if _descriptor._USE_C_DESCRIPTORS == False:

  DESCRIPTOR._options = None
  DESCRIPTOR._serialized_options = b'\n\030com.keepersecurity.protoB\005Vault'
  _CACHESTATUS._serialized_start=5820
  _CACHESTATUS._serialized_end=5854
  _SYNCDOWNREQUEST._serialized_start=76
  _SYNCDOWNREQUEST._serialized_end=120
  _SYNCDOWNRESPONSE._serialized_start=123
  _SYNCDOWNRESPONSE._serialized_end=2180
  _USERFOLDER._serialized_start=2183
  _USERFOLDER._serialized_end=2329
  _SHAREDFOLDER._serialized_start=2332
  _SHAREDFOLDER._serialized_end=2659
  _USERFOLDERSHAREDFOLDER._serialized_start=2661
  _USERFOLDERSHAREDFOLDER._serialized_end=2747
  _SHAREDFOLDERFOLDER._serialized_start=2750
  _SHAREDFOLDERFOLDER._serialized_end=2937
  _SHAREDFOLDERKEY._serialized_start=2939
  _SHAREDFOLDERKEY._serialized_end=3047
  _TEAM._serialized_start=3050
  _TEAM._serialized_end=3320
  _RECORD._serialized_start=3323
  _RECORD._serialized_end=3514
  _RECORDLINK._serialized_start=3516
  _RECORDLINK._serialized_end=3614
  _USERFOLDERRECORD._serialized_start=3616
  _USERFOLDERRECORD._serialized_end=3690
  _SHAREDFOLDERFOLDERRECORD._serialized_start=3692
  _SHAREDFOLDERFOLDERRECORD._serialized_end=3781
  _NONSHAREDDATA._serialized_start=3783
  _NONSHAREDDATA._serialized_end=3831
  _RECORDMETADATA._serialized_start=3834
  _RECORDMETADATA._serialized_end=4030
  _SHARINGCHANGE._serialized_start=4032
  _SHARINGCHANGE._serialized_end=4082
  _PROFILE._serialized_start=4084
  _PROFILE._serialized_end=4146
  _PROFILEPIC._serialized_start=4148
  _PROFILEPIC._serialized_end=4191
  _PENDINGTEAMMEMBER._serialized_start=4193
  _PENDINGTEAMMEMBER._serialized_end=4279
  _BREACHWATCHRECORD._serialized_start=4282
  _BREACHWATCHRECORD._serialized_end=4419
  _USERAUTH._serialized_start=4422
  _USERAUTH._serialized_end=4602
  _BREACHWATCHSECURITYDATA._serialized_start=4604
  _BREACHWATCHSECURITYDATA._serialized_end=4666
  _REUSEDPASSWORDS._serialized_start=4668
  _REUSEDPASSWORDS._serialized_end=4718
  _SHAREDFOLDERRECORD._serialized_start=4721
  _SHAREDFOLDERRECORD._serialized_end=4884
  _SHAREDFOLDERUSER._serialized_start=4887
  _SHAREDFOLDERUSER._serialized_end=5032
  _SHAREDFOLDERTEAM._serialized_start=5035
  _SHAREDFOLDERTEAM._serialized_end=5173
  _KSMCHANGE._serialized_start=5175
  _KSMCHANGE._serialized_end=5243
  _SHAREINVITATION._serialized_start=5245
  _SHAREINVITATION._serialized_end=5280
  _USER._serialized_start=5282
  _USER._serialized_end=5326
  _SYNCDIAGNOSTICS._serialized_start=5328
  _SYNCDIAGNOSTICS._serialized_end=5451
  _RECORDROTATION._serialized_start=5454
  _RECORDROTATION._serialized_end=5611
  _BREACHWATCHGETSYNCDATAREQUEST._serialized_start=5613
  _BREACHWATCHGETSYNCDATAREQUEST._serialized_end=5664
  _BREACHWATCHGETSYNCDATARESPONSE._serialized_start=5667
  _BREACHWATCHGETSYNCDATARESPONSE._serialized_end=5818
# @@protoc_insertion_point(module_scope)
